package com.ruoyi.web.controller.tool;

import java.util.List;
import javax.servlet.http.HttpServletResponse;

import com.ruoyi.system.domain.crm.CrmFile;
import com.ruoyi.system.service.crm.ICrmFileService;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.common.core.page.TableDataInfo;

/**
 * crm附件Controller
 * 
 * @author ruoyi
 * @date 2023-11-15
 */
@RestController
@RequestMapping("/crm/crmFile")
public class CrmFileController extends BaseController
{
    @Autowired
    private ICrmFileService crmFileService;

    /**
     * 查询crm附件列表
     */
    @PreAuthorize("@ss.hasPermi('crm:crmFile:list')")
    @GetMapping("/list")
    public TableDataInfo list(CrmFile crmFile)
    {
        startPage();
        List<CrmFile> list = crmFileService.selectCrmFileList(crmFile);
        return getDataTable(list);
    }

    /**
     * 导出crm附件列表
     */
    @PreAuthorize("@ss.hasPermi('crm:crmFile:export')")
    @Log(title = "crm附件", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, CrmFile crmFile)
    {
        List<CrmFile> list = crmFileService.selectCrmFileList(crmFile);
        ExcelUtil<CrmFile> util = new ExcelUtil<CrmFile>(CrmFile.class);
        util.exportExcel(response, list, "crm附件数据");
    }

    /**
     * 获取crm附件详细信息
     */
    @PreAuthorize("@ss.hasPermi('crm:crmFile:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return success(crmFileService.selectCrmFileById(id));
    }

    /**
     * 新增crm附件
     */
    @PreAuthorize("@ss.hasPermi('crm:crmFile:add')")
    @Log(title = "crm附件", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody CrmFile crmFile)
    {
        return toAjax(crmFileService.insertCrmFile(crmFile));
    }

    /**
     * 修改crm附件
     */
    @PreAuthorize("@ss.hasPermi('crm:crmFile:edit')")
    @Log(title = "crm附件", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody CrmFile crmFile)
    {
        return toAjax(crmFileService.updateCrmFile(crmFile));
    }

    /**
     * 删除crm附件
     */
    @PreAuthorize("@ss.hasPermi('crm:crmFile:remove')")
    @Log(title = "crm附件", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids)
    {
        return toAjax(crmFileService.deleteCrmFileByIds(ids));
    }

}
