package com.ruoyi.flowable.service.crm.Impl;

import com.alibaba.fastjson2.JSONObject;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.ruoyi.common.annotation.DataScope;
import com.ruoyi.common.config.RuoYiConfig;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.domain.entity.SysUser;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.common.utils.file.FileUtils;
import com.ruoyi.common.utils.uuid.IdUtils;
import com.ruoyi.flowable.common.enums.FlowComment;
import com.ruoyi.flowable.domain.dto.FlowTaskDto;
import com.ruoyi.flowable.domain.vo.FlowQueryVo;
import com.ruoyi.flowable.factory.FlowServiceFactory;
import com.ruoyi.flowable.service.crm.CrmService;
import com.ruoyi.system.domain.crm.Address;
import com.ruoyi.system.domain.crm.Contacts;
import com.ruoyi.system.domain.crm.CrmFile;
import com.ruoyi.system.domain.crm.Customer;
import com.ruoyi.system.service.crm.IAddressService;
import com.ruoyi.system.service.crm.IContactsService;
import com.ruoyi.system.service.crm.ICrmFileService;
import com.ruoyi.system.service.crm.ICustomerService;
import io.swagger.annotations.ApiParam;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.flowable.engine.repository.ProcessDefinition;
import org.flowable.engine.runtime.ProcessInstance;
import org.flowable.task.api.Task;
import org.flowable.task.api.TaskQuery;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.multipart.MultipartFile;
import com.ruoyi.flowable.common.constant.ProcessConstants;

import javax.annotation.Resource;
import java.io.File;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;

@Service
@Slf4j
public class CrmServiceImpl extends FlowServiceFactory implements CrmService {


    @Resource
    private ICustomerService customerService;

    @Resource
    private IContactsService contactsService;

    @Resource
    private IAddressService addressService;

    @Resource
    private ICrmFileService crmFileService;


    /**
     * 新增客户流程开启
     * @param procDefId
     * @param variables
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public AjaxResult startProcessInstanceById(String procDefId, Map<String, Object>  variables) {
        try {

            if (StringUtils.isNull(variables.get("customer")))return AjaxResult.error("请填写客户联系人信息");

            if (StringUtils.isNull(variables.get("contactsList")))return AjaxResult.error("请填写客户联系人信息");

            SysUser sysUser = SecurityUtils.getLoginUser().getUser();

            ProcessDefinition processDefinition = repositoryService.createProcessDefinitionQuery().processDefinitionId(procDefId)
                    .latestVersion().singleResult();
            if (Objects.nonNull(processDefinition) && processDefinition.isSuspended()) {
                return AjaxResult.error("流程已被挂起,请先激活流程");
            }

            identityService.setAuthenticatedUserId(sysUser.getUserId().toString());
            variables.put(ProcessConstants.PROCESS_INITIATOR, sysUser.getUserId());
            //设置部门领导审批人


            variables.put("MANAGER", sysUser.getUserId());
            //设置事业部负责人审批人
            variables.put("LEADER", sysUser.getUserId());
            //设置事业群领导审批人
            variables.put("DEPTLEADER", sysUser.getUserId());

            /**
             * todo 流程节点流转配置
             * =============================
             */



            //开启流程
            ProcessInstance processInstance = runtimeService.startProcessInstanceById(procDefId, variables);

            //判断审批通过或暂存
            //true 提交 false:暂存
            if(Boolean.valueOf(variables.get("FLAG").toString())) {
                // 给第一步申请人节点设置任务执行人和意见
                Task task = taskService.createTaskQuery().processInstanceId(processInstance.getProcessInstanceId()).singleResult();
                if (Objects.nonNull(task)) {
                    taskService.addComment(task.getId(), processInstance.getProcessInstanceId(), FlowComment.NORMAL.getType(), sysUser.getNickName() + "发起流程申请");
                    taskService.setAssignee(task.getId(), sysUser.getUserId().toString());
                    taskService.complete(task.getId(), variables);
                }
            }
            Customer customer = new ObjectMapper().readValue(variables.get("customer").toString(), Customer.class);

            List<Contacts> contactsList = new ObjectMapper().readValue(variables.get("contactsList").toString(), new TypeReference<List<Contacts>>(){});

            Address address = new ObjectMapper().readValue(variables.get("address").toString(), Address.class);

            //保存客户信息
            customer.setProcInsId(processInstance.getProcessInstanceId());
//			customer.setCustomerId(uuid);
//			customer.setCustomerCode(getNumber());
            customer.setCreateBy(sysUser.getUserName());
            customer.setSalesManager(sysUser.getUserName());
//            customer.setCompanyId(userInfo.getCompanyId());//所在的公司Id
            customer.setDeptId(sysUser.getDeptId().toString());//添加部门
            customer.setDatefrom("0");//系统
            if(Boolean.valueOf(variables.get("FLAG").toString())){
                customer.setState("0");//已提交状态
            }else{
                customer.setState("1");//暂存状态
            }
            customerService.insertCustomer(customer);

            //保存客户联系人信息
            if (CollectionUtils.isNotEmpty(contactsList)){
                SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMddHHmmss");
                for (Contacts contacts : contactsList) {
					contacts.setContactsId(IdUtils.fastUUID());
                    contacts.setCustomerId(customer.getCustomerId());
                    contacts.setContactCode(sdf.format(new Date()));
//                    contacts.setCompanyId(customer.getCompanyId());
                    contacts.setCreatedBy(sysUser.getUserName());
                }
                contactsService.batchInsertContacts(contactsList);
            }

            //保存账户信息
            if (StringUtils.isNull(address)){
                address.setCustomerId(customer.getCustomerId());
                addressService.insertAddress(address);
            }

            //保存附件信息
//            if(upfile.length>0){
//                this.uploadFile(upfile,variables.get("fileMap").toString());
//            }

            // todo 保存流程审批记录-待开发

            return AjaxResult.success("流程启动成功");
        } catch (Exception e) {
            e.printStackTrace();
            return AjaxResult.error("流程启动错误");
        }
    }

    public AjaxResult  uploadFile(MultipartFile[] upfile,String fileMap){
        try {
            SysUser sysUser = SecurityUtils.getLoginUser().getUser();
            if(sysUser!=null){
                List<CrmFile> fileList = new ArrayList<CrmFile>();
                JSONObject jsonObject = JSONObject.from(fileMap);
                Map<String, Map<String,List<String>>> mapStr = (Map)jsonObject;
                //获取文件保存地址
                String visit_url = RuoYiConfig.getUploadPath();

                for(int i=0;i<upfile.length;i++){
                    CrmFile cf = new CrmFile();
                    String fileName = upfile[i].getOriginalFilename().trim();

                    String uuid = IdUtils.fastUUID();
                    //获取文件后缀
                    String prefix=fileName.substring(fileName.lastIndexOf(".")+1);
                    String newFileName = uuid+"."+prefix;
                    File file = new File(visit_url+"/crmFile");
                    if(!file.exists()){
                        file.mkdirs();
                    }
                    upfile[i].transferTo(new File(file,newFileName));
                    cf.setFileId(IdUtils.fastUUID());
                    List<String> data = FileUtils.getData(mapStr, fileName);
                    if(data!=null && data.size()>0 && data.size()>1){
                        cf.setObjectId(data.get(0));
                        cf.setType(data.get(1));
                    }
                    cf.setUploadName(fileName);
                    cf.setUploadPath(visit_url+"/crmFile/"+newFileName);
                    cf.setUploadType(prefix);
                    fileList.add(cf);
                }
                if(fileList.size()>0){
                    for (CrmFile po:fileList) {
                        crmFileService.insertCrmFile(po);
                    }
                }
                return AjaxResult.success("文件上传成功");
            }else{
                return AjaxResult.error("系统出错，请联系管理员");
            }
        } catch (Exception e) {
            e.printStackTrace();
            return AjaxResult.error("文件上传失败");
        }

    }


}
