package com.ruoyi.system.service.resume.impl;

import java.util.Date;
import java.util.List;
import java.util.Objects;

import com.ruoyi.system.domain.resume.*;
import com.ruoyi.system.mapper.resume.ResumeMapper;
import com.ruoyi.system.service.resume.*;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

/**
 * 简历管理Service业务层处理
 * 
 * @author ruoyi
 * @date 2024-02-18
 */
@Service
public class ResumeServiceImpl implements IResumeService
{
    @Autowired
    private ResumeMapper resumeMapper;

    @Autowired
    private IResumeFileService resumeFileService;

    @Autowired
    private IResumeWorkService resumeWorkService;

    @Autowired
    private IResumeProjectService resumeProjectService;

    @Autowired
    private IResumeEducationService resumeEducationService;

    /**
     * 查询简历管理
     * 
     * @param resumeId 简历管理主键
     * @return 简历管理
     */
    @Override
    public Resume selectResumeByResumeId(Long resumeId)
    {
        Resume resume = resumeMapper.selectResumeByResumeId(resumeId);
        if(!Objects.isNull(resume)){
            //简历附件处理
            ResumeFile resumeFile = new ResumeFile();
            resumeFile.setResumeId(resume.getResumeId());
            List<ResumeFile> resumeFileList = resumeFileService.selectResumeFileList(resumeFile);
            if(resumeFileList != null && !resumeFileList.isEmpty()){
                resume.setResumeFile(resumeFileList.get(0));
            }
            //简历工作履历处理
            ResumeWork work = new ResumeWork();
            work.setResumeId(resume.getResumeId());
            List<ResumeWork> workList = resumeWorkService.selectResumeWorkList(work);
            resume.setWorkList(workList);
            //简历项目经验处理
            ResumeProject project = new ResumeProject();
            project.setResumeId(resume.getResumeId());
            List<ResumeProject> projectList = resumeProjectService.selectResumeProjectList(project);
            resume.setProjectList(projectList);
            //简历教育经历处理
            ResumeEducation education = new ResumeEducation();
            education.setResumeId(resume.getResumeId());
            List<ResumeEducation> eduList = resumeEducationService.selectResumeEducationList(education);
            resume.setEduList(eduList);
        }
        return resume;
    }

    /**
     * 查询简历管理列表
     * 
     * @param resume 简历管理
     * @return 简历管理
     */
    @Override
    public List<Resume> selectResumeList(Resume resume)
    {
        return resumeMapper.selectResumeList(resume);
    }

    /**
     * 新增简历管理
     * 
     * @param resume 简历管理
     * @return 结果
     */
    @Override
    public int insertResume(Resume resume)
    {
        resume.setCreateDate(new Date());
        int row = resumeMapper.insertResume(resume);
        if(row > 0){
            //简历附件处理
            if(!Objects.isNull(resume.getResumeFile())){
                ResumeFile resumeFile = resume.getResumeFile();
                resumeFile.setResumeId(resume.getResumeId());
                resumeFileService.insertResumeFile(resumeFile);
            }
            //简历工作履历处理
            List<ResumeWork> workList = resume.getWorkList();
            if(workList != null && !workList.isEmpty()){
                for(ResumeWork work : workList){
                    work.setResumeId(resume.getResumeId());
                    resumeWorkService.insertResumeWork(work);
                }
            }
            //简历项目经验处理
            List<ResumeProject> projectList = resume.getProjectList();
            if(projectList != null && !projectList.isEmpty()){
                for(ResumeProject project : projectList){
                    project.setResumeId(resume.getResumeId());
                    resumeProjectService.insertResumeProject(project);
                }
            }
            //简历教育经历处理
            List<ResumeEducation> eduList = resume.getEduList();
            if(eduList != null && !eduList.isEmpty()){
                for(ResumeEducation education : eduList){
                    education.setResumeId(resume.getResumeId());
                    resumeEducationService.insertResumeEducation(education);
                }
            }
        }
        return row;
    }

    /**
     * 修改简历管理
     * 
     * @param resume 简历管理
     * @return 结果
     */
    @Override
    public int updateResume(Resume resume) {
        int row = resumeMapper.updateResume(resume);
        if(row > 0){
            //简历附件处理
            if(!Objects.isNull(resume.getResumeFile())){
                resumeFileService.deleteResumeFileByResumeId(resume.getResumeId());
                ResumeFile resumeFile = resume.getResumeFile();
                resumeFile.setResumeId(resume.getResumeId());
                resumeFileService.insertResumeFile(resumeFile);
            }
            //简历工作履历处理
            List<ResumeWork> workList = resume.getWorkList();
            if(workList != null && !workList.isEmpty()){
                //删除之前的简历工作履历
                resumeWorkService.deleteResumeWorkByResumeId(resume.getResumeId());
                for(ResumeWork work : workList){
                    work.setResumeId(resume.getResumeId());
                    resumeWorkService.insertResumeWork(work);
                }
            }
            //简历项目经验处理
            List<ResumeProject> projectList = resume.getProjectList();
            if(projectList != null && !projectList.isEmpty()){
                //删除之前的简历项目经验
                resumeProjectService.deleteResumeProjectByResumeId(resume.getResumeId());
                for(ResumeProject project : projectList){
                    project.setResumeId(resume.getResumeId());
                    resumeProjectService.insertResumeProject(project);
                }
            }
            //简历教育经历处理
            List<ResumeEducation> eduList = resume.getEduList();
            if(eduList != null && !eduList.isEmpty()){
                //删除之前的简历教育经历
                resumeEducationService.deleteResumeEducationByResumeId(resume.getResumeId());
                for(ResumeEducation education : eduList){
                    education.setResumeId(resume.getResumeId());
                    resumeEducationService.insertResumeEducation(education);
                }
            }
        }
        return row;
    }

    /**
     * 批量删除简历管理
     * 
     * @param resumeIds 需要删除的简历管理主键
     * @return 结果
     */
    @Override
    public int deleteResumeByResumeIds(Long[] resumeIds)
    {
        return resumeMapper.deleteResumeByResumeIds(resumeIds);
    }

    /**
     * 删除简历管理信息
     * 
     * @param resumeId 简历管理主键
     * @return 结果
     */
    @Override
    public int deleteResumeByResumeId(Long resumeId)
    {
        return resumeMapper.deleteResumeByResumeId(resumeId);
    }
}
